--------------------------------------------------------------------------------
-- 
-- Name: WhoLibGlobals.lua
-- Author: Malex-Cenarius
-- Date: 12/23/2006
--
--------------------------------------------------------------------------------


-- Stores internally used global data for the addon
--
WhoLib = {};

-- Stores internally used function pointers for the addon
--
WhoLib.Util = {};

-- Stores constants used internally for the addon
--
WhoLib.Constants = {
	CacheVersion = 2,					-- Used to invalidate a cache data file if the format changes
	IsDebug = false, 					-- Enables debug output and settings
	UseTestData = false, 				-- For testing, grab who information for anybody that talks
	TestQuery = false,					-- Turns on testing of the WhoLib_Query function
	TestCallbacks = false,				-- Tests the callback parameters
	ProcessQueueFrequency = 500,		-- The frequency that internal ProcessQueue is executed in milliseconds
	TimeoutQueryResultsFrequency = 5,	-- The frequency that the query results are polled for timeout
	WhoQueueTimeout = 30,   			-- The amount of time that a request can exist in the queue before timing out in seconds
	WhoRequestTimeout = 3000,			-- The amount of time before a pending who request expires in milliseconds	
	MaxTimestamp = 1800,				-- The default value in seconds for GetUserData requests for which data will be discarded as stale
	MaxCachedEntries = 5000, 			-- The cap cache size at which point the entire cache is flushed (should be 1MB)
	QueryResultsTimeout = 60,			-- The number of seconds that query results will remain in the results table without being retrieved		
	MaxRetries = 3						-- The number of retries that a query has before returning an empty results table
}

-- Stores the cached who data
--
WhoLibCache = {};

-- Stores the list of usernames that require a who request
--
WhoLib.Queue = {};
WhoLib.QueueSize = 0; -- The size of the queue
WhoLib.QueueFilterList = {};

-- A table to hold query results until they are retrieved
--
WhoLib.QueryResults = {};
WhoLib.QueryResults.n = 0;

-- The query for which the current who request is pending
--
WhoLib.QueuePendingQuery = nil;

-- The time that has elapsed since the current who request started in milliseconds
--
WhoLib.QueuePendingElapsed = 0;

-- Specifies whether the current who request is a complex filter
--
WhoLib.QueuePendingIsComplex = false;

-- The function pointer for ChatFrame_OnEvent used to remove background who request
-- messages from the chat log
--
WhoLib.ChatFrame_OnEvent_Original = nil;

-- This function pointer for the FriendsFrame_OnEvent is used to intercept
-- WHO_LIST_UPDATE messages during complex queries so that the who popup
-- doesn't show
--
WhoLib.FriendsFrame_OnEvent_Original = nil;
WhoLib.FriendsFrame_OnShow_Original = nil;
WhoLib.FriendsFrame_OnHide_Original = nil;

WhoLib.FriendsFrameIsVisible = false;

WhoLib.DebugQueries = {
	"c-\"Warrior\"",
	"c-\"Paladin\"",
	"c-\"Rogue\"",
	"c-\"Mage\"",
	"c-\"Priest\"",
};
