--------------------------------------------------------------------------------
-- 
-- Name: WhoLib.lua
-- Author: Malex-Cenarius
-- Date: 12/23/2006
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--
-- Method: DebugOut
-- Description: A simple text output method for debugging
--
--------------------------------------------------------------------------------
local function DebugOut(text)

	if(WhoLib.Constants.IsDebug) then
		DEFAULT_CHAT_FRAME:AddMessage(text);		
	end
	
end

--------------------------------------------------------------------------------
--
-- Method: Out
-- Description: A simple text output method
--
--------------------------------------------------------------------------------
local function Out(text)
	
	DEFAULT_CHAT_FRAME:AddMessage(text);			
	
end


--------------------------------------------------------------------------------
--
-- Method: IsWhoFrameVisible
-- Description: Returns true of the who frame is visible, otherwise false
--
--------------------------------------------------------------------------------
local function IsWhoFrameVisible()

	local whoFrame = getglobal("WhoFrame");
	if(whoFrame:IsVisible()) then
		return true;
	else
		return false;
	end

end

--------------------------------------------------------------------------------
--
-- Method: ContainsSystemWhoText
-- Description: Returns true if the text parameter contains any system who
--	result text
--
--------------------------------------------------------------------------------
local function ContainsSystemWhoText(text) 

	if(text == format(WHO_NUM_RESULTS, 1)) then
	
		return true;
		
	end
		
	if(string.find(text, string.sub(WHO_NUM_RESULTS_P1, 3)) ~= nil) then
		
		return true;
	
	end
	
	return false;

end

--------------------------------------------------------------------------------
--
-- Method: RemoveFilterFromQueue
-- Description: 
--
--------------------------------------------------------------------------------
local function RemoveFilterFromQueue(filter)

	for index, value in ipairs(WhoLib.QueueFilterList) do
	
		if(value == filter) then					
			tremove(WhoLib.QueueFilterList, index);
			return;
		end
	
	end

end

--------------------------------------------------------------------------------
--
-- Method: MoveWhoRequestForward
-- Description: This method is called when a user-initiated who request is processed
--	and needs to be moved before any other requests
--
--------------------------------------------------------------------------------
local function MoveWhoRequestForward(userName)

	-- Don't do anything if it's already at the front of the queue
	--
	if(WhoLib.QueueFilterList[1] == userName) then
		return;	
	end
	
	-- Remove the userName from the filter list
	--
	for i = 0, getn(WhoLib.QueueFilterList) do
		if(userName == WhoLib.QueueFilterList[i]) then
			tremove(WhoLib.QueueFilterList, i);
			break;
		end
	end
		
	-- Add the userName back into the filter list either first
	-- or right after the pending query
	--
	if(WhoLib.QueuePendingQuery == nil) then
		tinsert(WhoLib.QueueFilterList, 1, userName);
	else
		tinsert(WhoLib.QueueFilterList, 2, userName);
	end

end

--------------------------------------------------------------------------------
--
-- Method: AddCallbackToQueue
-- Description: 
--
--------------------------------------------------------------------------------
local function AddCallbackToQueue(filter, callback, context)

	if(WhoLib.Queue[filter] ~= nil and callback ~= nil) then
	
		-- Figure out if the callback should be added
		--
		local addCallback = true;
		
		if(WhoLib.Queue[filter].Callbacks ~= nil) then
			
			for index, item in ipairs(WhoLib.Queue[filter].Callbacks) do
			
				if(item.Callback == callback) then
					addCallback = false;
				end
			
			end
		
		end
		
		-- Add the callback to the queued item 
		--
		if(addCallback) then
				
			local callbackPair = {
				Callback = callback,
				Context = context
			};
		
			tinsert(WhoLib.Queue[filter].Callbacks, callbackPair);
		
		end
	
	
	end

end

--------------------------------------------------------------------------------
--
-- Method: GetUserInfoCallback
-- Description: 
--
--------------------------------------------------------------------------------
local function GetUserInfoCallback(data, context)

	if(data ~= nil) then
	
		if(data.Name ~= context) then
			WhoLib.Util.DebugOut("Invalid callback parameters");
		end
	
		local secondsPassed = time() - data.Timestamp;
		local friendlyTimePassed = SecondsToTime(secondsPassed);
		
		WhoLib.Util.DebugOut("UserDataFoundCallback: Name:" .. data.Name .. " " .. data.GuildName .. " " .. data.Level .. " " .. data.Race .. " " .. data.Class .. " " .. data.Zone .. " " .. friendlyTimePassed);

	else
	
		WhoLib.Util.DebugOut("|cFFFF0000UserDataNOTfound: " .. context);
	
	end

end

--------------------------------------------------------------------------------
--
-- Method: GetUserInfoCallback2
-- Description: 
--
--------------------------------------------------------------------------------
local function GetUserInfoCallback2(data, context)

	if(data ~= nil) then
	
		if(data.Name ~= context) then
			WhoLib.Util.DebugOut("Invalid callback parameters2");
		end
	
		local secondsPassed = time() - data.Timestamp;
		local friendlyTimePassed = SecondsToTime(secondsPassed);
		
		WhoLib.Util.DebugOut("UserDataFoundCallback2: Name:" .. data.Name .. " " .. data.GuildName .. " " .. data.Level .. " " .. data.Race .. " " .. data.Class .. " " .. data.Zone .. " " .. friendlyTimePassed);

	else
	
		WhoLib.Util.DebugOut("|cFFFF0000UserDataNOTfound2: " .. context);
	
	end

end

--------------------------------------------------------------------------------
--
-- Method: QueryResultsCallback
-- Description: 
--
--------------------------------------------------------------------------------
local function QueryResultsCallback(data, context)

	if(data ~= nil) then
	
		WhoLib.Util.DebugOut("QueryDataFoundCallback: " .. getn(data) .. " entries for query: " .. context);
	
	else
		
		WhoLib.Util.DebugOut("|cFFFF0000QueryNOTfound: " .. context);
		
	end

end



WhoLib.Util = {
	DebugOut = DebugOut,
	Out = Out,
	IsWhoFrameVisible = IsWhoFrameVisible,
	ContainsSystemWhoText = ContainsSystemWhoText,
	RemoveFilterFromQueue = RemoveFilterFromQueue,
	MoveWhoRequestForward = MoveWhoRequestForward,
	AddCallbackToQueue = AddCallbackToQueue,
	GetUserInfoCallback = GetUserInfoCallback,
	GetUserInfoCallback2 = GetUserInfoCallback2,
	QueryResultsCallback = QueryResultsCallback
}


